from cba.utils.eval_utils import (
    compute_rouge_score,
    compute_g_eval,
    compute_llama3_eval,
)


def score_insight(gt_insight, pred_insight, score_name):
    """
    Get the scoring function based on the score_name.

    Returns:
    --------
    score (float): The score of the prediction based on the ground truth.
    """
    if score_name == "rouge1":
        score = compute_rouge_score(pred_insight, gt_insight)
    elif score_name == "g_eval":
        score = compute_g_eval(pred_insight, gt_insight, top_logprobs=5) / 10.0
    elif score_name == "llama3_eval":
        score = compute_llama3_eval(pred_insight, gt_insight, top_logprobs=5) / 10.0
    else:
        raise ValueError(f"Unknown score_name: {score_name}")

    # score has to be between 0 and 1
    assert score >= 0 and score <= 1

    return score


if __name__ == "__main__":
    import json
    from cba.utils import agent_utils as au

    scores_ = []
    summary_scores_ = []
    for i in [1, 2, 3, 4, 6, 7]:
        print(f"Flag {i}")
        pred_insights = json.load(
            open(f"/mnt/home/research-cba/outputs/copilot_ih{i}.json")
        )
        gt_insights = au.analysis_nb_to_gt(
            f"/mnt/home/research-cba/data/notebooks/flag-{i}.ipynb"
        )
        scores = []
        for pred_insight in pred_insights["insights"]:
            scores.append(
                score_insight(
                    pred_insight,
                    [o["insight_dict"]["insight"] for o in gt_insights["insights"]],
                    score_name="llama3_eval",
                )
            )
        summary_score = score_insight(
            pred_insights["summary"], gt_insights["flag"], score_name="llama3_eval"
        )
        import numpy as np

        print(f"insight-level score {i}: {np.mean(scores)}")
        print(f"summary-level score {i}: {summary_score}")

        scores_.append(scores)
        summary_scores_.append(summary_score)
    print(f"insight-level scores: {np.mean(scores_)}")
    print(f"summary-level scores: {np.mean(summary_scores_)}")
