import pytest
import tempfile
import json
import os
import pandas as pd

from cba.utils import agent_utils as au
from cba.agents import Agent
from cba.utils import metrics_utils as mu


def test_gt_notebook():
    datadir = "data/notebooks"
    fname_ipynb = fname_ipynb = os.path.join(datadir, "flag-1.ipynb")
    gt_insights_dict = au.analysis_nb_to_gt(fname_ipynb)
    dataset_csv_path = os.path.join(datadir, gt_insights_dict["dataset_csv_path"])
    df = pd.read_csv(dataset_csv_path)
    with tempfile.TemporaryDirectory() as tmp_dir:
        agent = Agent(
            df=df,
            df_user=None,
            gen_engine="dummy",
            max_questions=2,
            branch_depth=5,
            n_retries=2,
            savedir=tmp_dir,
        )

        pred_insights_dict = agent.run_agent()
        print()

    # Ensure 100%
    gt_flag_dict = au.analysis_nb_to_gt(fname_ipynb)
    gt_insights = []
    for o in gt_flag_dict["insights"]:
        gt_insights += [o["insight_dict"]["insight"]]

    pred_insights_dict_as_gt = {
        0: [{"answer": {"answer": insight}} for insight in gt_insights]
    }
    score_name = "rouge1"
    gt_summary = gt_flag_dict["flag"]
    scores = mu.score_insight(gt_summary, gt_summary, score_name=score_name)

    assert scores == 1.0

    # Ensure 0%
    scores = mu.score_insight(gt_summary, "wrwetwrwr", score_name=score_name)

    assert scores == 0.0

    # Ensure that it works
    scores, score_summary = agent.evaluate_agent_on_summary(
        gt_flag_dict, score_name="rouge1", return_summary=True
    )
    # scores is a number
    assert isinstance(scores, float)

    # score_summary is a dict that has "score_summary", pred_summary and gt_summary
    assert isinstance(score_summary, dict)
    assert "score_summary" in score_summary
    assert "pred_summary" in score_summary
    assert "gt_summary" in score_summary


if __name__ == "__main__":
    test_gt_notebook()
