import pytest
import tempfile
import json
import os
import pandas as pd

from cba.utils import agent_utils as au


def test_gt_notebook():
    datadir = "data/notebooks"
    fname_ipynb = fname_ipynb = os.path.join(datadir, "flag-1.ipynb")
    gt_json = au.analysis_nb_to_gt(fname_ipynb, include_df_head=True)
    dataset_csv_path = os.path.join(datadir, gt_json["dataset_csv_path"])
    # replace anything behind data with empty string
    assert os.path.exists(dataset_csv_path)
    df = pd.read_csv(dataset_csv_path).head()

    for i in range(len(df)):
        print(i)
        df_cell = df.iloc[i]
        gt_cell = gt_json["df_head"][0].iloc[i]
        for c in df.columns:
            if not str(df_cell[c])[:5] == str(gt_cell[c])[:5]:
                raise ValueError(f"df_cell: {df_cell[c]} != gt_cell: {gt_cell[c]}")
    del gt_json["df_head"]
    with tempfile.TemporaryDirectory() as tmp_dir:

        fname_json = os.path.join(
            tmp_dir, os.path.split(fname_ipynb)[-1].replace(".ipynb", ".json")
        )

        with open(fname_json, "w") as f:
            json.dump(gt_json, f, indent=4)

        assert os.path.exists(fname_json)


if __name__ == "__main__":
    test_gt_notebook()
