import numpy as np
import pytest
from langchain.schema import SystemMessage

from cba.utils.agent_utils import extract_html_tags, chat_and_retry


def test_extract_html_tags_single_occurrence():
    text = "<p>This is a paragraph.</p>"
    assert extract_html_tags(text, ["p"])["p"] == ["This is a paragraph."]


def test_extract_html_tags_multiple_occurrences():
    html_text = "<div><p>First paragraph.</p><p>Second paragraph.</p></div>"
    assert extract_html_tags(html_text, ["p"])["p"] == [
        "First paragraph.",
        "Second paragraph.",
    ]


def test_extract_html_tags_tag_not_found():
    text = "<p>This is a paragraph.</p>"
    assert extract_html_tags(text, ["span"]).get("span", None) is None


def test_extract_html_tags_empty_text():
    text = ""
    assert extract_html_tags(text, ["p"]).get("p", None) is None


def test_extract_html_tags_empty_tag():
    text = "<p>This is a paragraph.</p>"
    assert len(extract_html_tags(text, [])) == 0


def test_extract_html_tags_nested_tags():
    text = "<div><p>Outer paragraph.<p>Nested paragraph.</p></div>"
    assert extract_html_tags(text, ["p"])["p"] == [
        "Outer paragraph.<p>Nested paragraph."
    ]


# @pytest.mark.pricy
# def test_chat_and_retry():
#     chat = ChatOpenAI(model_name="gpt-3.5-turbo", temperature=0.2, n=3)
#     prompt = """List primes from 1 to 10."""
#     messages = [
#         SystemMessage(content=prompt),
#     ]

#     global n_call
#     n_call = 0

#     def parser(message):
#         global n_call
#         n_call += 1

#         if n_call <= 3:  # First 3 calls, just answer the new prompt
#             err_message = "I changed my mind. List primes up to 15. Just answer the json list, nothing else."
#             return None, False, err_message, 0

#         elif n_call == 5:
#             return "success", True, "", 10
#         else:
#             return "bad", True, "", 1 + np.random.rand()

#     value = chat_and_retry(chat, messages, parser=parser, n_retry=2)
#     assert value == "success"
#     assert n_call == 6  # 2*3 calls


if __name__ == "__main__":
    test_extract_html_tags_empty_tag()
    test_extract_html_tags_empty_text()
    test_extract_html_tags_single_occurrence()
    test_extract_html_tags_multiple_occurrences()
    test_extract_html_tags_tag_not_found()
    test_extract_html_tags_nested_tags()
    # test_chat_and_retry()
