import pytest, os
import pandas as pd
import tempfile

from cba.agents import Agent
from collections import namedtuple
from pathlib import Path
from cba.utils import agent_utils as au

path_prefix = Path(__file__).parent


def test_agent():
    datadir = "data/notebooks"
    fname_ipynb = fname_ipynb = os.path.join(datadir, "flag-1.ipynb")
    gt_insights_dict = au.analysis_nb_to_gt(fname_ipynb)
    dataset_csv_path = os.path.join(datadir, gt_insights_dict["dataset_csv_path"])

    df = pd.read_csv(dataset_csv_path)
    with tempfile.TemporaryDirectory() as tmp_dir:

        agent = Agent(
            df=df,
            df_user=None,
            savedir=tmp_dir,
            gen_engine="dummy",
            max_questions=1,
            branch_depth=5,
            n_retries=2,
        )

        pred_insights_dict = agent.run_agent()
    if 1:
        assert agent.max_questions == 1
        assert agent.gen_engine == "dummy"
        assert agent.n_retries == 2
        assert agent.branch_depth == 5

    assert len(pred_insights_dict) == agent.max_questions * agent.branch_depth


if __name__ == "__main__":
    test_agent()
